@@ -4,13 +4,16 @@ Constants: KickGroups
 
 Constants used in the KickGroups
 """
+
 from pathlib import Path
 
 KICKGROUPS_ROOT = Path("/user/slops/data/LHC_DATA/OP_DATA/Betabeat/KickGroups/MULTITURN_ACQ_GROUPS")
 
 # Columns ---
 KICKGROUP = "KICKGROUP"
+JSON_FILE = "JSON"
 SDDS = "SDDS"
+FILL = "FILL"
 TURNS = "TURNS"
 BUNCH = "BUNCH"
 UTCTIME = "UTC"
@@ -29,6 +32,26 @@ OPTICS_URI = "OPTICS_URI"
 BEAMPROCESS = "BEAMPROCESS"
 BEAM = "BEAM"
 
-KICK_COLUMNS = [UTCTIME, LOCALTIME, TUNEX, TUNEY, DRIVEN_TUNEX, DRIVEN_TUNEY, DRIVEN_TUNEZ, AMPX, AMPY, AMPZ, TURNS, BUNCH, SDDS, BEAM, OPTICS, OPTICS_URI, BEAMPROCESS]
-COLUMNS_TO_HEADERS = [BEAM, BUNCH, TURNS, BEAMPROCESS, OPTICS, OPTICS_URI]
-KICK_GROUP_COLUMNS = [UTCTIME, LOCALTIME, KICKGROUP, TIMESTAMP]
\ No newline at end of file
+KICK_COLUMNS = [
+    UTCTIME,
+    LOCALTIME,
+    TUNEX,
+    TUNEY,
+    DRIVEN_TUNEX,
+    DRIVEN_TUNEY,
+    DRIVEN_TUNEZ,
+    AMPX,
+    AMPY,
+    AMPZ,
+    TURNS,
+    BUNCH,
+    SDDS,
+    JSON_FILE,
+    BEAM,
+    FILL,
+    OPTICS,
+    OPTICS_URI,
+    BEAMPROCESS,
+]
+COLUMNS_TO_HEADERS = [BEAM, FILL, BUNCH, TURNS, BEAMPROCESS, OPTICS, OPTICS_URI]
+KICK_GROUP_COLUMNS = [UTCTIME, LOCALTIME, KICKGROUP, TIMESTAMP]
