@@ -4,23 +4,25 @@ PyLSA
 
 This module provides useful functions to conveniently wrap the functionality of ``pjlsa``.
 """
-import jpype
+
 import logging
 import re
+from collections.abc import Callable
+
+import jpype
 import tfs
-from jpype import java, JException
+from jpype import JException, java
 from omc3.utils.mock import cern_network_import
 from omc3.utils.time_tools import AccDatetime
-from typing import Callable, Union, Dict, Tuple, List
 
 LOG = logging.getLogger(__name__)
 pytimber = cern_network_import("pytimber")
 pjlsa = cern_network_import("pjlsa")
 
 try:
-    pjLSAClient = pjlsa.LSAClient
+    pjLSAClient = pjlsa.LSAClient  # noqa: N816 (it's the real name)
 except ImportError:
-    pjLSAClient = object
+    pjLSAClient = object  # noqa: N816 (it's the real name)
 
 RELEVANT_BP_CONTEXTS = ("OPERATIONAL", "MD")
 RELEVANT_BP_CATEGORIES = ("DISCRETE",)
@@ -39,7 +41,7 @@ class LSAClient(pjLSAClient):
     """Extension of the LSAClient."""
 
     def __getattr__(self, item):
-        """ Overwrite __getattr__ to raise the proper import errors at the proper time."""
+        """Overwrite __getattr__ to raise the proper import errors at the proper time."""
         try:
             super().__getattr__(item)
         except AttributeError as e:
@@ -59,14 +61,19 @@ class LSAClient(pjLSAClient):
         Returns:
             Sorted list of knob names.
         """
+        LOG.debug(f"Getting knobs for {accelerator}.")
         req = self._ParametersRequestBuilder()
         req.setAccelerator(self._getAccelerator(accelerator))
         req.setParameterTypeName("KNOB")
         lst = self._parameterService.findParameters(req.build())
-        reg = re.compile(regexp, re.IGNORECASE)
-        return sorted(filter(reg.search, [pp.getName() for pp in lst]))
+        LOG.debug(f"{len(lst)} Knobs extracted for {accelerator}.")
+        if regexp:
+            LOG.debug(f"Selecting Knobs containing expression: {regexp}")
+            reg = re.compile(regexp, re.IGNORECASE)
+            return sorted(filter(reg.search, [pp.getName() for pp in lst]))
+        return sorted(pp.getName() for pp in lst)
 
-    def find_existing_knobs(self, knobs: List[str]) -> List[str]:
+    def find_existing_knobs(self, knobs: list[str]) -> list[str]:
         """
         Return only the knobs that exist from the given list.
         This function was created out of the need to filter these first,
@@ -80,6 +87,7 @@ class LSAClient(pjLSAClient):
         Returns:
             A list of the knob names that actually exist.
         """
+        LOG.debug(f"Checking if the following knobs exist: {knobs}")
         dont_exist = [k for k in knobs if self._getParameter(k) is None]
         if len(dont_exist):
             LOG.warning(f"The following knobs do not exist and will be filtered: {dont_exist}.")
@@ -87,8 +95,8 @@ class LSAClient(pjLSAClient):
         return knobs
 
     def find_last_fill(
-            self, acc_time: AccDatetime, accelerator: str = "lhc", source: str = "nxcals"
-    ) -> Tuple[str, list]:
+        self, acc_time: AccDatetime, accelerator: str = "lhc", source: str = "nxcals"
+    ) -> tuple[str, list]:
         """
         Return last fill name and BeamProcesses.
 
@@ -99,11 +107,12 @@ class LSAClient(pjLSAClient):
 
         Returns:
             tuple: Last fill name (str), Beamprocesses of last fill (list).
-         """
+        """
         start_time = acc_time.sub(days=1)  # assumes a fill is not longer than a day
         try:
             fills = self.find_beamprocess_history(
-                t_start=start_time, t_end=acc_time,
+                t_start=start_time,
+                t_end=acc_time,
                 accelerator=accelerator,
                 source=source,
             )
@@ -115,8 +124,12 @@ class LSAClient(pjLSAClient):
         return last_fill, fills[last_fill]
 
     def find_beamprocess_history(
-            self, t_start: AccDatetime, t_end: AccDatetime, accelerator: str = "lhc", source: str = "nxcals"
-    ) -> Dict:
+        self,
+        t_start: AccDatetime,
+        t_end: AccDatetime,
+        accelerator: str = "lhc",
+        source: str = "nxcals",
+    ) -> dict:
         """
         Finds the BeamProcesses between t_start and t_end and sorts then by fills.
         Adapted from pjlsa's FindBeamProcessHistory but with source pass-through
@@ -132,27 +145,37 @@ class LSAClient(pjLSAClient):
             Dictionary of fills (keys) with a list of Timestamps and BeamProcesses.
 
         """
-        cts = self.findUserContextMappingHistory(t_start.timestamp(), t_end.timestamp(), accelerator=accelerator)
+        cts = self.findUserContextMappingHistory(
+            t_start.timestamp(), t_end.timestamp(), accelerator=accelerator
+        )
 
-        db = pytimber.LoggingDB(source=source)
+        db = pytimber.LoggingDB(source=source, loglevel=logging.WARNING)
         fillnts, fillnv = try_to_acquire_data(
             db.get, "HX:FILLN", t_start.timestamp(), t_end.timestamp()
         )["HX:FILLN"]
 
         if not len(fillnv):
-            raise ValueError(f"No beamprocesses for {accelerator} ({source}) found between {t_start} - {t_end}.")
+            raise ValueError(
+                f"No beamprocesses for {accelerator} ({source}) found between {t_start} - {t_end}."
+            )
 
+        LOG.debug(f"{len(fillnts)} fills aqcuired.")
+        # map beam-processes to fills
         fills = {}
         for ts, name in zip(cts.timestamp, cts.name):
             idx = fillnts.searchsorted(ts) - 1
             filln = int(fillnv[idx])
             fills.setdefault(filln, []).insert(0, (ts, name))
+        LOG.debug("Beamprocess History extracted.")
         return fills
 
     def get_trim_history(
-            self, beamprocess: str, knobs: list, 
-            start_time: AccDatetime = None, end_time: AccDatetime = None, 
-            accelerator: str = "lhc"           
+        self,
+        beamprocess: str,
+        knobs: list,
+        start_time: AccDatetime = None,
+        end_time: AccDatetime = None,
+        accelerator: str = "lhc",
     ) -> dict:
         """
         Get trim history for knobs between specified times.
@@ -169,6 +192,7 @@ class LSAClient(pjLSAClient):
         Returns:
             Dictionary of trims and their data (as TrimTuples, i.e. NamedTuple of lists of time and data).
         """
+        LOG.debug("Extracting Trim history.")
         if knobs is None or len(knobs) == 0:
             knobs = self.find_knob_names(accelerator)
         else:
@@ -177,25 +201,33 @@ class LSAClient(pjLSAClient):
                 raise ValueError("None of the given knobs exist!")
 
         if start_time is not None:
-            start_time = start_time.timestamp() 
-        
-        if end_time is not None:
-            end_time = end_time.timestamp() 
+            start_time = start_time.timestamp()
 
+        if end_time is not None:
+            end_time = end_time.timestamp()
+
+        LOG.debug(f"Getting trims for {len(knobs)} knobs.")
         try:
-            trims = self.getTrims(parameter=knobs, beamprocess=beamprocess, start=start_time, end=end_time)
+            trims = self.getTrims(
+                parameter=knobs, beamprocess=beamprocess, start=start_time, end=end_time
+            )
         except jpype.java.lang.NullPointerException as e:
             # In the past this happened, when a knob was not defined, but
             # this should have been caught by the filter_existing_knobs above
-            raise ValueError(f"Something went wrong when extracting trims for the knobs: {knobs}") from e
+            raise ValueError(
+                f"Something went wrong when extracting trims for the knobs: {knobs}"
+            ) from e
 
-        trims_not_found = [k for k in knobs if k not in trims.keys()]
+        LOG.debug(f"{len(trims)} trims extracted.")
+        trims_not_found = [k for k in knobs if k not in trims]
         if len(trims_not_found):
-            LOG.warning(f"The following knobs were not found in '{beamprocess}': {trims_not_found}")
+            LOG.warning(
+                f"The following knobs were not found in '{beamprocess}' "
+                f"or had no trims during the given time: {trims_not_found}"
+            )
         return trims
 
-
-    def get_beamprocess_info(self, beamprocess: Union[str, object]) -> Dict:
+    def get_beamprocess_info(self, beamprocess: str | object) -> dict:
         """
         Get context info of the given beamprocess.
 
@@ -206,14 +238,17 @@ class LSAClient(pjLSAClient):
             Dictionary with context info.
         """
         if isinstance(beamprocess, str):
+            LOG.debug(f"Extracting Beamprocess {beamprocess} from ContextService")
             beamprocess = self._contextService.findStandAloneBeamProcess(beamprocess)
         bp_dict = _beamprocess_to_dict(beamprocess)
-        LOG.debug(str(bp_dict))
+        LOG.debug(f"Beamprocess details: {str(bp_dict)}")
         return bp_dict
 
     def find_active_beamprocess_at_time(
-            self, acc_time: AccDatetime, accelerator: str = "lhc",
-            bp_group: str = "POWERCONVERTERS"  # the Beamprocesses relevant for OMC,
+        self,
+        acc_time: AccDatetime,
+        accelerator: str = "lhc",
+        bp_group: str = "POWERCONVERTERS",  # the Beamprocesses relevant for OMC,
     ):
         """
         Find the active beam process at the time given.
@@ -236,8 +271,10 @@ class LSAClient(pjLSAClient):
         )
         beamprocess = beamprocessmap.get(bp_group)
         if beamprocess is None:
-            raise ValueError(f"No active BeamProcess found for group '{bp_group}' "
-                             f"at time {acc_time.utc_string} UTC.")
+            raise ValueError(
+                f"No active BeamProcess found for group '{bp_group}' "
+                f"at time {acc_time.utc_string} UTC."
+            )
         LOG.debug(f"Active Beamprocess at time '{acc_time.cern_utc_string()}': {str(beamprocess)}")
         return beamprocess
 
@@ -245,7 +282,7 @@ class LSAClient(pjLSAClient):
         """
         Get a dataframe of the structure of the knob. Similar to online model extractor
         (KnobExtractor.getKnobHiercarchy)
-        
+
         Args:
             knob_name: name of the knob.
             optics: name of the optics.
@@ -260,11 +297,11 @@ class LSAClient(pjLSAClient):
         df.headers[HEAD_INFO] = "In MAD-X it should be 'name = name + DELTA * knobValue'"
         knob = self._knobService.findKnob(knob_name)
         if knob is None:
-            raise IOError(f"Knob '{knob_name}' does not exist")
+            raise OSError(f"Knob '{knob_name}' does not exist")
         try:
             knob_settings = knob.getKnobFactors().getFactorsForOptic(optics)
         except jpype.java.lang.IllegalArgumentException:
-            raise IOError(f"Knob '{knob_name}' not available for optics '{optics}'")
+            raise OSError(f"Knob '{knob_name}' not available for optics '{optics}'")
 
         for knob_factor in knob_settings:
             factor = knob_factor.getFactor()
@@ -286,11 +323,15 @@ class LSAClient(pjLSAClient):
     def get_madx_name_from_circuit(self, circuit: str):
         """Returns the ``MAD-X`` Strength Name (Circuit/Knob) from the given circuit name."""
         logical_name = circuit.split("/")[0]
-        slist = jpype.java.util.Collections.singletonList(
+        slist = jpype.java.util.Collections.singletonList(  # python lists did not work (jdilly)
             logical_name
-        )  # python lists did not work (jdilly)
-        madx_name = self._deviceService.findMadStrengthNamesByLogicalNames(slist)  # returns a map
-        return madx_name[logical_name]
+        )
+        madx_name_map = self._deviceService.findMadStrengthNamesByLogicalNames(
+            slist
+        )  # returns a map
+        madx_name = madx_name_map[logical_name]
+        LOG.debug(f"Name conversion: {circuit} -> {logical_name} -> {madx_name}")
+        return madx_name
 
 
 # Single Instance LSAClient ####################################################
@@ -311,7 +352,6 @@ class LSAMeta(type):
 
             def hooked(*args, **kwargs):
                 result = client_attr(*args, **kwargs)
-                result_is_self = False
                 try:
                     if result == cls._client:
                         # prevent client from becoming unwrapped
@@ -323,14 +363,14 @@ class LSAMeta(type):
                 return result
 
             return hooked
-        else:
-            return client_attr
+        return client_attr
 
 
 class LSA(metaclass=LSAMeta):
     """Import this class to use LSA like the client without the need to instantiate it.
     Disadvantage: It will always use the default Server.
     """
+
     pass
 
 
@@ -339,10 +379,14 @@ class LSA(metaclass=LSAMeta):
 
 def _beamprocess_to_dict(bp):
     """Converts some fields of the beamprocess (java) to a dictionary."""
-    bp_dict = {'Name': bp.toString(), "Object": bp}
-    bp_dict.update({getter[3:]: str(bp.__getattribute__(getter)())  # __getattr__ does not exist
-                    for getter in dir(bp)
-                    if getter.startswith('get') and "Attribute" not in getter})
+    bp_dict = {"Name": bp.toString(), "Object": bp}
+    bp_dict.update(
+        {
+            getter[3:]: str(bp.__getattribute__(getter)())  # __getattr__ does not exist
+            for getter in dir(bp)
+            if getter.startswith("get") and "Attribute" not in getter
+        }
+    )
     return bp_dict
 
 
@@ -357,15 +401,16 @@ def try_to_acquire_data(function: Callable, *args, **kwargs):
     Returns:
         Return arguments of ``function``
 
-     """
+    """
     retries = MAX_RETRIES
     for tries in range(retries + 1):
         try:
             return function(*args, **kwargs)
         except java.lang.IllegalStateException as e:
-            raise IOError("Could not acquire data, user probably has no access to NXCALS") from e
+            raise OSError("Could not acquire data, user probably has no access to NXCALS") from e
         except JException as e:  # Might be a case for retries
             if "RetryableException" in str(e) and (tries + 1) < retries:
                 LOG.warning(f"Could not acquire data! Trial no {tries + 1} / {retries}")
                 continue  # will go to the next iteratoin of the loop, so retry
-            raise IOError("Could not acquire data!") from e
+            raise OSError("Could not acquire data!") from e
+    raise RuntimeError(f"Could not acquire data after {retries:d} retries.")
